@@ -1,10 +1,11 @@
 /* Reading PO files.
-   Copyright (C) 1995-1996, 1998, 2000-2006 Free Software Foundation, Inc.
+   Copyright (C) 1995-1996, 1998, 2000-2006, 2015-2016 Free Software
+   Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -25,11 +25,17 @@
 #include "po-lex.h"
 #include "po-gram.h"
 
+/* For compiling this file in C++ mode.  */
+#ifdef __cplusplus
+# define this thiss
+#endif
+
+
 /* Read a .po / .pot file from a stream, and dispatch to the various
    abstract_catalog_reader_class_ty methods.  */
 static void
 po_parse (abstract_catalog_reader_ty *this, FILE *fp,
-	  const char *real_filename, const char *logical_filename)
+          const char *real_filename, const char *logical_filename)
 {
   lex_start (fp, real_filename, logical_filename);
   po_gram_parse ();
@@ -38,6 +44,6 @@ po_parse (abstract_catalog_reader_ty *this, FILE *fp,
 
 const struct catalog_input_format input_format_po =
 {
-  po_parse,				/* parse */
-  false					/* produces_utf8 */
+  po_parse,                             /* parse */
+  false                                 /* produces_utf8 */
 };
