@@ -1,5 +1,5 @@
 /* List of exported symbols of libgettextlib on Cygwin.
-   Copyright (C) 2006-2007 Free Software Foundation, Inc.
+   Copyright (C) 2006-2007, 2010, 2015-2016 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2006.
 
    This program is free software: you can redistribute it and/or modify
@@ -18,15 +18,19 @@
 #include "woe32dll/export.h"
 
 VARIABLE(argmatch_die)
-VARIABLE(error_message_count)
-VARIABLE(error_one_per_line)
-VARIABLE(error_print_progname)
+#if GNULIB_DEFINED_ERROR
+VARIABLE(rpl_error_message_count)
+VARIABLE(rpl_error_one_per_line)
+VARIABLE(rpl_error_print_progname)
+#endif
 VARIABLE(error_with_progname)
 VARIABLE(exit_failure)
 VARIABLE(gl_array_list_implementation)
 VARIABLE(gl_linkedhash_list_implementation)
 VARIABLE(program_name)
+#if GNULIB_DEFINED_GETOPT
 VARIABLE(rpl_optarg)
 VARIABLE(rpl_optind)
+#endif
 VARIABLE(simple_backup_suffix)
 VARIABLE(styled_ostream_typeinfo)
