@@ -1,5 +1,6 @@
 /* List of exported symbols of libgettextsrc on Cygwin.
-   Copyright (C) 2006-2007, 2009, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2006-2007, 2009, 2011, 2015-2016 Free Software Foundation,
+   Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2006.
 
    This program is free software: you can redistribute it and/or modify
@@ -15,6 +16,10 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "woe32dll/export.h"
 
 VARIABLE(allow_duplicates)
@@ -32,6 +37,7 @@ VARIABLE(formatstring_gfc_internal)
 VARIABLE(formatstring_java)
 VARIABLE(formatstring_javascript)
 VARIABLE(formatstring_kde)
+VARIABLE(formatstring_kde_kuit)
 VARIABLE(formatstring_librep)
 VARIABLE(formatstring_lisp)
 VARIABLE(formatstring_lua)
@@ -56,7 +62,6 @@ VARIABLE(input_format_po)
 VARIABLE(input_format_properties)
 VARIABLE(input_format_stringtable)
 VARIABLE(less_than)
-VARIABLE(line_comment)
 VARIABLE(more_than)
 VARIABLE(msgcomm_mode)
 VARIABLE(omit_header)
@@ -72,11 +77,14 @@ VARIABLE(po_error)
 VARIABLE(po_error_at_line)
 VARIABLE(po_gram_lval)
 VARIABLE(po_lex_charset)
+#if HAVE_ICONV
 VARIABLE(po_lex_iconv)
+#endif
 VARIABLE(po_lex_weird_cjk)
 VARIABLE(po_multiline_error)
 VARIABLE(po_multiline_warning)
 VARIABLE(po_xerror)
 VARIABLE(po_xerror2)
+VARIABLE(syntax_check_name)
 VARIABLE(style_file_name)
 VARIABLE(use_first)
